const Clock = require('common/clock.js');
const BCD = require('utils/bcd.js');
const BITM = require('utils/bitman.js');

function ChangeICBit(cia, v, pos){
  let mask = ~(1 << pos);
  v = ((v === true || v === 1) ? 1 : 0) << pos;
  cia.__IC = (cia.__IC & mask) | v;
}


function TODTick(cia){
  cia.__TODTick += 1;
  if (cia.__TODTick === 6){
    cia.__TODTick = cia.__CTA & 0x01;
    if (cia.__TOD[0] == 0x09){
      cia.__TOD[0] = 0;
      if (cia.__TOD[1] == 0x59){
        cia.__TOD[1] = 0;
        if (cia.__TOD[2] == 0x59){
          cia.__TOD[2] = 0;
          if (cia.__TOD[3] == 0x11 || cia.__TOD[3] == 0x91){
            cia.__TOD[3] = (cia.__TOD[3] == 0x91) ? 0x00 : 0x10;
          } else {
            let pm = cia.__TOD[3] & 0x80;
            cia.__TOD[3] = BCD.add(cia.__TOD[3] & 0x7F, 0x01, 2) | pm;
          }
        } else {cia.__TOD[2] = BCD.add(cia.__TOD[2], 0x01, 2);}
      } else {cia.__TOD[1] = BCD.add(cia.__TOD[1], 0x01, 2);}
    } else {cia.__TOD[0] = BCD.add(cia.__TOD[0], 0x01, 2);}
  }

  if (cia.__TOD[0] == cia.__ALTOD[0] && cia.__TOD[1] == cia.__ALTOD[1] && cia.__TOD[2] == cia.__ALTOD[2] && cia.__TOD[3] == cia.__ALTOD[3]){
    ChangeICBit(cia, 1, 2);
  }
}

function TODLatch(cia){
  if (cia.__TODLatch === 0){
    cia.__TODLatch = 1;
    cia.__LTOD[0] = cia.__TOD[0];
    cia.__LTOD[1] = cia.__TOD[1];
    cia.__LTOD[2] = cia.__TOD[2];
    cia.__LTOD[3] = cia.__TOD[3];
  }
}


class MOSCIA{
  constructor(){
    this.__regsel = 0;

    this.__PDA = 0;
    this.__PDB = 0;
    this.__DDA = 0;
    this.__DDB = 0;

    this.__CTA = 0;
    this.__CTB = 0;
    this.__IC = 0;

    this.__timerA = 0;
    this.__timerALatch = 0;
    this.__timerB = 0;
    this.__timerBLatch = 0;

    // Time Of Day
    this.__TODLatch = 0;
    this.__TODTick = 0;
    this.__TOD = [0,0,0,0];
    this.__LTOD = [0,0,0,0];
    this.__ALTOD = [0,0,0,0];
  }

  get RS(){return this.__regsel;}
  set RS(rs){this.__regsel = rs & 0x0F;}

  get DATA(){
    let val = 0;
    switch(this.__regsel){
      case 0x00: // Peripheral Data A
        break;
      case 0x01: // Peripheral Data B
        break;
      case 0x02: // Data Direction Reg A
        break;
      case 0x03: // Data Direction Reg B
        break;
      case 0x04: // Timer A Low
        break;
      case 0x05: // Timer A High
        break
      case 0x06: // Timer B Low
        break;
      case 0x07: // Timer B High
        break;
      case 0x08: // 10th of Sec Reg
        this.__TODLatch = 0;
        return this.__TOD[0]; break;
      case 0x09: // Seconds Reg
        return (this.__TODLatch > 0) ? this.__LTOD[1] : this.__TOD[1]; break;
      case 0x0A: // Minutes Reg
        return (this.__TODLatch > 0) ? this.__LTOD[2] : this.__TOD[2]; break;
      case 0x0B: // Hours AM/PM Reg
        TODLatch(this);
        return this.__LTOD[3]; break;
      case 0x0C: // Serial Data
        break;
      case 0x0D: // Interrupt Control
        break;
      case 0x0E: // Control Timer Reg A
        break;
      case 0x0F: // Control Timer Reg B
        break;
    }
    return val;
  }

  set DATA(d){
    switch(this.__regsel){
      case 0x00: // Peripheral Data A
        break;
      case 0x01: // Peripheral Data B
        break;
      case 0x02: // Data Direction Reg A
        break;
      case 0x03: // Data Direction Reg B
        break;
      case 0x04: // Timer A Low
        break;
      case 0x05: // Timer A High
        break
      case 0x06: // Timer B Low
        break;
      case 0x07: // Timer B High
        break;
      case 0x08: // 10th of Sec Reg
        TODLatch(this);
        this.__TODLatch = 2;
        let tod = (BITM.isOn(this.__CTB, 7)) ? this.__ALTOD : this.__TOD;
        tod[0] = d & 0xFF;
        break;
      case 0x09: // Seconds Reg
        TODLatch(this);
        this.__TODLatch = 2;
        let tod = (BITM.isOn(this.__CTB, 7)) ? this.__ALTOD : this.__TOD;
        tod[1] = d & 0xFF;
        break;
      case 0x0A: // Minutes Reg
        TODLatch(this);
        this.__TODLatch = 2;
        let tod = (BITM.isOn(this.__CTB, 7)) ? this.__ALTOD : this.__TOD;
        tod[2] = d & 0xFF;
        break;
      case 0x0B: // Hours AM/PM Reg
        TODLatch(this);
        this.__TODLatch = 2;
        let tod = (BITM.isOn(this.__CTB, 7)) ? this.__ALTOD : this.__TOD;
        tod[3] = d & 0xFF;
        break;
      case 0x0C: // Serial Data
        break;
      case 0x0D: // Interrupt Control
        break;
      case 0x0E: // Control Timer Reg A
        break;
      case 0x0F: // Control Timer Reg B
        break;
    }
  }

  get CNT(){return 0;}
  set CNT(c){}

  get SP(){return 0;}
  set SP(b){}

  get TOD(){return 0;}
  set TOD(b){
    if (b > 0 || this.__TODLatch < 2)
      TODTick(this); 
  }
}


module.exports = MOSCIA;


